// @flow
import React from 'react';
import { expect } from 'chai';
import { mount } from 'enzyme';
import sinon from 'sinon';
import { Provider } from 'react-redux';
import { spies } from '@test';

import { getValidToolchainObj as getValidObj, getValidDatabaseObj, getValidAlgorithmObj } from '@helpers/beat';
import { ToolchainEditor as C } from '.';
import * as Selectors from '@store/selectors';
import createStoreFunc from '@store/store';
import reducer from '@store/reducers';

import testTcs from '@test/test_tcs.json';
import testDbs from '@test/test_dbs.json';
import testAlgs from '@test/test_algs.json';

describe('<ToolchainEditor />', () => {
	let wrapper;

	afterEach(() => {
		if(wrapper && wrapper.unmount)
			wrapper.unmount();
	});

	describe('accepts', () => {
		const tcs = testTcs.map(tc => getValidObj(tc));
		const dbs = testDbs.map(db => getValidDatabaseObj(db));
		const algs = testAlgs.map(alg => getValidAlgorithmObj(alg));

		const state = {
			...reducer({}, { type: '', payload: {}}),
			toolchain: tcs,
			database: dbs,
			algorithm: algs,
		};

		const sets = Selectors.flattenedDatabases(state);
		const normalAlgorithms = Selectors.normalBlocks(state);
		const analyzerAlgorithms = Selectors.analyzerBlocks(state);
		const store = createStoreFunc(state);

		tcs.forEach(function(tc){
			const saveFunc = () => {};
			const updateFunc = () => {};
			it(`${ tc.name }`, () => {
				wrapper = mount(
					<Provider store={store}>
						<C
							data={tc}
							sets={sets}
							toolchains={state.toolchain}
							databases={state.database}
							normalAlgorithms={normalAlgorithms}
							analyzerAlgorithms={analyzerAlgorithms}
							saveFunc={saveFunc}
							updateFunc={updateFunc}
						/>
					</Provider>
				);

				expect(wrapper.find(C)).to.have.props(
					['data', 'toolchains', 'databases', 'normalAlgorithms', 'analyzerAlgorithms', 'saveFunc', 'updateFunc']
				).deep.equal(
					[tc, tcs, state.database, normalAlgorithms, analyzerAlgorithms, saveFunc, updateFunc]
				);
			});
		});
	});

	describe('creates', () => {

	});
});
