// @flow
import * as React from 'react';
import {
	Container,
	Row,
	Col,
	Button,
	ButtonGroup,
	Form,
	FormGroup,
	Label,
	Input,
	InputGroup,
	FormText,
	Collapse,
	Card,
	CardHeader,
	CardBody,
	TabContent, TabPane,
	Navbar,
	Nav, NavItem, NavLink, CardTitle, CardText,
	FormFeedback,
	Alert,
	InputGroupAddon,
	Badge,
	Modal, ModalBody, ModalHeader, ModalFooter,
	UncontrolledButtonDropdown, DropdownToggle, DropdownMenu, DropdownItem,
} from 'reactstrap';

import CacheInput from '../CacheInput.jsx';
import DeleteInputBtn from '../DeleteInputBtn.jsx';
import { generateNewKey } from '@helpers';

type Props = {
	// block data
	data: any,
	// func to close modal
	toggle: () => any,
	// is the modal active?
	active: boolean,
	// name of all blocks
	blockNames: string[],
	// change the block name
	updateBlockName: (newName: string) => any,
	// change the name of an input or output
	updateBlockIOName: (ioType: 'input' | 'output', oldName: string, newName: string) => any,
	// create an input or output
	addBlockIO: (ioType: 'input' | 'output', newName: string) => any,
	// delete an input or output
	deleteBlockIO: (ioType: 'input' | 'output', ioName: string) => any,
	// delete the block
	deleteBlock: () => any,
	// possible channels the block can be sync'd to (an obj w keys as channel names & vals as colours)
	possibleChannels: any,
	// update sync'd channel
	updateBlockChannel: (channel: string) => any,
};

type State = {
};

class ToolchainModal extends React.Component<Props, State> {
	constructor(props: Props){
		super(props);
	}

	state = {
	}

	shouldComponentUpdate(nextProps: Props){
		if(nextProps.active !== this.props.active)
			return true;
		if(nextProps.data !== undefined && nextProps.data !== this.props.data)
			return true;
		return false;
	}

	render = () => {
		const data = this.props.data;
		if(!data)
			return null;

		const hasInputs = Array.isArray(data.inputs);
		const hasOutputs = Array.isArray(data.outputs);
		// TODO: right now this is just an escape hatch for channel changing not propogating throughout the network. this needs to be fixed
		const channelSelectDisabled = false; //Object.keys(this.props.possibleChannels).length === 1;

		return (
			<Modal
				isOpen={this.props.active}
				toggle={this.props.toggle}
				fade={true}
				onOpened={e => {
					const f = document.querySelector('#tcModalInitFocus');
					if(f && f.focus)
						f.focus();
				}}
			>
				<ModalHeader toggle={this.props.toggle}>
					Editing <pre className='preInline'>{data.name}</pre>
					{' '}
				</ModalHeader>
				<ModalBody>
					<Form>
						<FormGroup>
							<CacheInput
								delay={200}
								id='tcModalInitFocus'
								value={data.name || ''}
								fieldTest={true}
								onChange={(e) => {
									this.props.updateBlockName(e.target.value);
								}}
								validateFunc={(str) => {
									return (
										str === data.name ||
										!this.props.blockNames.includes(str) ||
										<span>This name is already taken by an existing block</span>
									);
								}}
							/>
						</FormGroup>
						{ data.hasOwnProperty('synchronized_channel')
							&&
							<FormGroup>
								<Label>Channel: {data.synchronized_channel}</Label>
								<Input
									type='select'
									disabled={channelSelectDisabled}
									value={data.synchronized_channel}
									onChange={e => this.props.updateBlockChannel(e.target.value)}
									title={channelSelectDisabled ? 'Cannot change the synchronized channel because there is not more than one to choose from' : undefined}
								>
									{ /* TODO: adding the current value in the list of possible values is part of the escape hatch for channel changes not propogating throughout the network. this needs to be fixed */ }
									{
										Array.from(new Set([...Object.keys(this.props.possibleChannels), data.synchronized_channel]))
										.map((channel, i) =>
											<option
												key={i}
												value={channel}
											>
												{ channel }
											</option>
										)
									}
								</Input>
							</FormGroup>
						}
						<Row>
							{
								hasInputs &&
									<Col sm={hasOutputs ? 6 : 12}>
										Inputs
										{
											data.inputs.map((input, i, inputs) =>
												<FormGroup
													key={i}
												>
													<CacheInput
														value={input}
														fieldTest={true}
														onChange={(e) => {
															this.props.updateBlockIOName('input', input, e.target.value);
														}}
														validateFunc={(str) => {
															return (
																str === input ||
																!inputs.includes(str) ||
																<span>This input name is already in use in this block</span>
															);
														}}
													>
														<DeleteInputBtn
															deleteFunc={e => {
																this.props.deleteBlockIO('input', input);
															}}
														/>
													</CacheInput>
												</FormGroup>
											)
										}
										<Button
											onClick={e => {
												const newKey = generateNewKey('input', data.inputs);
												this.props.addBlockIO('input', newKey);
											}}
										>
											Add Input
										</Button>
									</Col>
							}
							{
								hasOutputs &&
									<Col sm={hasInputs ? 6 : 12}>
										Outputs
										{
											data.outputs.map((output, i, outputs) =>
												<FormGroup
													key={i}
												>
													<CacheInput
														value={output}
														fieldTest
														onChange={(e) => {
															this.props.updateBlockIOName('output', output, e.target.value);
														}}
														validateFunc={(str) => {
															return (
																str === output ||
																!outputs.includes(str) ||
																<span>This output name is already in use in this block</span>
															);

														}}
													>
														<DeleteInputBtn
															deleteFunc={e => {
																this.props.deleteBlockIO('output', output);
															}}
														/>
													</CacheInput>
												</FormGroup>
											)
										}
										<Button
											onClick={e => {
												const newKey = generateNewKey('output', data.outputs);
												this.props.addBlockIO('output', newKey);
											}}
										>
											Add Output
										</Button>
									</Col>
							}
						</Row>
					</Form>
				</ModalBody>
				<ModalFooter>
					<Button block color='danger'
						onClick={(e) => {
							this.props.deleteBlock();
							this.props.toggle();
						}}
					>
						Delete Block
					</Button>
				</ModalFooter>
			</Modal>
		);
	}
}

export default ToolchainModal;
